@@ -23,6 +23,25 @@
 #define Py_TPFLAGS_HAVE_WEAKREFS  0
 #endif
 
+#if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION < 5
+/* Py_ssize_t was introduced in Python 2.5, substitute long for it */
+typedef long Py_ssize_t;
+#define PY_SSIZE_T_MAX  LONG_MAX
+#define PY_SSIZE_T_MIN  LONG_MIN
+Py_ssize_t PyNumber_AsSsize_t(PyObject *o, PyObject *exc)
+{
+    return PyLong_AsLong(o);
+}
+int PyIndex_Check(PyObject *o)
+{
+    return 0;
+}
+#define PY_SSIZE_T_FMT  "l"
+#else
+/* Python 2.5 and up uses 'n' as the format char for Py_ssize_t */
+#define PY_SSIZE_T_FMT  "n"
+#endif
+
 #if PY_MAJOR_VERSION == 2 && PY_MINOR_VERSION < 6
 /* backward compatibility with Python 2.5 */
 #define Py_TYPE(ob)   (((PyObject *) (ob))->ob_type)
@@ -215,25 +234,31 @@ copy_n(bitarrayobject *self, idx_t a,
     assert(0 <= n && n <= self->nbits && n <= other->nbits);
     assert(0 <= a && a <= self->nbits - n);
     assert(0 <= b && b <= other->nbits - n);
+    if (n == 0) {
+        return;
+    }
 
-    /* XXX
     if (self->endian == other->endian && a % 8 == 0 && b % 8 == 0 && n >= 8)
     {
-        Py_ssize_t bytes;
-        idx_t bits;
+        const Py_ssize_t bytes = (Py_ssize_t) n / 8;
+        const idx_t bits = bytes * 8;
 
-        bytes = n / 8;
-        bits = 8 * bytes;
-        copy_n(self, bits + a, other, bits + b, n - bits);
-        memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
+        if (a <= b) {
+            memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
+        }
+        if (n != bits) {
+            copy_n(self, bits + a, other, bits + b, n - bits);
+        }
+        if (a > b) {
+            memmove(self->ob_item + a / 8, other->ob_item + b / 8, bytes);
+        }
         return;
     }
-    */
 
     /* the different type of looping is only relevant when other and self
        are the same object, i.e. when copying a piece of an bitarrayobject
        onto itself */
-    if (a < b) {
+    if (a <= b) {
         for (i = 0; i < n; i++)             /* loop forward (delete) */
             setbit(self, i + a, GETBIT(other, i + b));
     }
@@ -402,32 +427,38 @@ bytereverse(bitarrayobject *self)
     }
 }
 
+
+static int bitcount_lookup[256] = {
+    0, 1, 1, 2, 1, 2, 2, 3, 1, 2, 2, 3, 2, 3, 3, 4,
+    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
+    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
+    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
+    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
+    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
+    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
+    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
+    1, 2, 2, 3, 2, 3, 3, 4, 2, 3, 3, 4, 3, 4, 4, 5,
+    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
+    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
+    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
+    2, 3, 3, 4, 3, 4, 4, 5, 3, 4, 4, 5, 4, 5, 5, 6,
+    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
+    3, 4, 4, 5, 4, 5, 5, 6, 4, 5, 5, 6, 5, 6, 6, 7,
+    4, 5, 5, 6, 5, 6, 6, 7, 5, 6, 6, 7, 6, 7, 7, 8,
+};
+
 /* returns number of 1 bits */
 static idx_t
 count(bitarrayobject *self)
 {
-    static int bitcount[256];
-    static int setup = 0;
     Py_ssize_t i;
     idx_t res = 0;
     unsigned char c;
 
-    if (!setup) {
-        /* setup lookup table, which maps each byte to it's bit count:
-           bitcount = {0, 1, 1, 2, 1, 2, 2, 3, 1, ..., 8} */
-        int j, k;
-        for (k = 0; k < 256; k++) {
-            bitcount[k] = 0;
-            for (j = 0; j < 8; j++)
-                bitcount[k] += (k >> j) & 1;
-        }
-        setup = 1;
-    }
-
     setunused(self);
     for (i = 0; i < Py_SIZE(self); i++) {
         c = self->ob_item[i];
-        res += bitcount[c];
+        res += bitcount_lookup[c];
     }
     return res;
 }
@@ -545,15 +576,22 @@ static int
 extend_bitarray(bitarrayobject *self, bitarrayobject *other)
 {
     idx_t n_sum;
+    idx_t n_other_bits;
 
     if (other->nbits == 0)
         return 0;
 
+    /*
+        Note: other may be self. Thus we take the size before we resize,
+        ensuring we only copy the right parts of the array.
+    */
+    n_other_bits = other->nbits;
     n_sum = self->nbits + other->nbits;
+
     if (resize(self, n_sum) < 0)
         return -1;
 
-    copy_n(self, n_sum - other->nbits, other, 0, other->nbits);
+    copy_n(self, n_sum - n_other_bits, other, 0, n_other_bits);
     return 0;
 }
 
@@ -736,7 +774,7 @@ extend_dispatch(bitarrayobject *self, PyObject *obj)
 
 /* --------- helper functions NOT involving bitarrayobjects ------------ */
 
-#define ENDIANSTR(x)  ((x) ? "big" : "little")
+#define ENDIAN_STR(ba)  (((ba)->endian) ? "big" : "little")
 
 #ifdef IS_PY3K
 #define IS_INDEX(x)  (PyLong_Check(x) || PyIndex_Check(x))
@@ -1018,7 +1056,7 @@ bitarray_search(bitarrayobject *self, PyObject *args)
     bitarrayobject *xa;
     idx_t p;
 
-    if (!PyArg_ParseTuple(args, "O|n:_search", &x, &limit))
+    if (!PyArg_ParseTuple(args, "O|" PY_SSIZE_T_FMT ":_search", &x, &limit))
         return NULL;
 
     if (!bitarray_Check(x)) {
@@ -1073,7 +1111,7 @@ bitarray_buffer_info(bitarrayobject *self)
     res = Py_BuildValue("OLsiL",
                         ptr,
                         (idx_t) Py_SIZE(self),
-                        ENDIANSTR(self->endian),
+                        ENDIAN_STR(self),
                         (int) (BITS(Py_SIZE(self)) - self->nbits),
                         (idx_t) self->allocated);
     Py_DECREF(ptr);
@@ -1094,9 +1132,9 @@ static PyObject *
 bitarray_endian(bitarrayobject *self)
 {
 #ifdef IS_PY3K
-    return PyUnicode_FromString(ENDIANSTR(self->endian));
+    return PyUnicode_FromString(ENDIAN_STR(self));
 #else
-    return PyString_FromString(ENDIANSTR(self->endian));
+    return PyString_FromString(ENDIAN_STR(self));
 #endif
 }
 
@@ -1177,7 +1215,7 @@ bitarray_reduce(bitarrayobject *self)
         goto error;
     PyMem_Free((void *) str);
     result = Py_BuildValue("O(Os)O", Py_TYPE(self),
-                           repr, ENDIANSTR(self->endian), dict);
+                           repr, ENDIAN_STR(self), dict);
 error:
     Py_DECREF(dict);
     Py_XDECREF(repr);
@@ -1242,7 +1280,7 @@ PyDoc_STRVAR(fill_doc,
 "fill() -> int\n\
 \n\
 Adds zeros to the end of the bitarray, such that the length of the bitarray\n\
-is not a multiple of 8.  Returns the number of bits added (0..7).");
+will be a multiple of 8.  Returns the number of bits added (0..7).");
 
 
 static PyObject *
@@ -1406,7 +1444,7 @@ bitarray_fromfile(bitarrayobject *self, PyObject *args)
     idx_t t, p;
     long cur;
 
-    if (!PyArg_ParseTuple(args, "O|n:fromfile", &f, &nbytes))
+    if (!PyArg_ParseTuple(args, "O|" PY_SSIZE_T_FMT ":fromfile", &f, &nbytes))
         return NULL;
 
     fp = PyFile_AsFile(f);
@@ -1548,8 +1586,9 @@ bitarray_tolist(bitarrayobject *self)
         return NULL;
 
     for (i = 0; i < self->nbits; i++)
-        PyList_SetItem(list, (Py_ssize_t) i,
-                       PyBool_FromLong(GETBIT(self, i)));
+        if (PyList_SetItem(list, (Py_ssize_t) i,
+                           PyBool_FromLong(GETBIT(self, i))) < 0)
+            return NULL;
     return list;
 }
 
@@ -2506,24 +2545,23 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 {
     PyObject *a;  /* to be returned in some cases */
     PyObject *initial = NULL;
-    char *endianStr = "<NOT_PROVIDED>";
+    char *endian_str = NULL;
     int endian;
     static char* kwlist[] = {"initial", "endian", NULL};
 
     if (!PyArg_ParseTupleAndKeywords(args, kwds,
-                                     "|Os:bitarray", kwlist,
-                                     &initial, &endianStr)) {
+                        "|Os:bitarray", kwlist, &initial, &endian_str))
         return NULL;
+
+    if (endian_str == NULL) {
+        endian = DEFAULT_ENDIAN;  /* use default value */
     }
-    if (strcmp(endianStr, "little") == 0) {
+    else if (strcmp(endian_str, "little") == 0) {
         endian = 0;
     }
-    else if (strcmp(endianStr, "big") == 0) {
+    else if (strcmp(endian_str, "big") == 0) {
         endian = 1;
     }
-    else if (strcmp(endianStr, "<NOT_PROVIDED>") == 0) {
-        endian = DEFAULT_ENDIAN;  /* use default value */
-    }
     else {
         PyErr_SetString(PyExc_ValueError,
                         "endian must be 'little' or 'big'");
@@ -2552,8 +2590,7 @@ bitarray_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
     if (bitarray_Check(initial)) {
 #define np  ((bitarrayobject *) initial)
         a = newbitarrayobject(type, np->nbits,
-                              (strcmp(endianStr, "<NOT_PROVIDED>") == 0  ?
-                               np->endian : endian));
+                              endian_str == NULL ? np->endian : endian);
         if (a == NULL)
             return NULL;
         memcpy(((bitarrayobject *) a)->ob_item, np->ob_item, Py_SIZE(np));
@@ -2901,6 +2938,47 @@ static PyTypeObject Bitarraytype = {
 
 /*************************** Module functions **********************/
 
+static PyObject *
+bitdiff(PyObject *self, PyObject *args)
+{
+    PyObject *a, *b;
+    Py_ssize_t i;
+    idx_t res = 0;
+    unsigned char c;
+
+    if (!PyArg_ParseTuple(args, "OO:bitdiff", &a, &b))
+        return NULL;
+    if (!(bitarray_Check(a) && bitarray_Check(b))) {
+        PyErr_SetString(PyExc_TypeError, "bitarray object expected");
+        return NULL;
+    }
+
+#define aa  ((bitarrayobject *) a)
+#define bb  ((bitarrayobject *) b)
+    if (aa->nbits != bb->nbits) {
+        PyErr_SetString(PyExc_ValueError,
+                        "bitarrays of equal length expected");
+        return NULL;
+    }
+    setunused(aa);
+    setunused(bb);
+    for (i = 0; i < Py_SIZE(aa); i++) {
+        c = aa->ob_item[i] ^ bb->ob_item[i];
+        res += bitcount_lookup[c];
+    }
+#undef aa
+#undef bb
+    return PyLong_FromLongLong(res);
+}
+
+PyDoc_STRVAR(bitdiff_doc,
+"bitdiff(a, b) -> int\n\
+\n\
+Return the difference between two bitarrays a and b.\n\
+This is function does the same as (a ^ b).count(), but is more memory\n\
+efficient, as no intermediate bitarray object gets created");
+
+
 static PyObject *
 bits2bytes(PyObject *self, PyObject *v)
 {
@@ -2919,7 +2997,6 @@ bits2bytes(PyObject *self, PyObject *v)
     return PyLong_FromLongLong(BYTES(n));
 }
 
-
 PyDoc_STRVAR(bits2bytes_doc,
 "bits2bytes(n) -> int\n\
 \n\
@@ -2948,8 +3025,9 @@ tuple(sizeof(void *),\n\
 
 
 static PyMethodDef module_functions[] = {
-    {"bits2bytes", (PyCFunction) bits2bytes, METH_O,      bits2bytes_doc},
-    {"_sysinfo",   (PyCFunction) sysinfo,    METH_NOARGS, sysinfo_doc   },
+    {"bitdiff",    (PyCFunction) bitdiff,    METH_VARARGS, bitdiff_doc   },
+    {"bits2bytes", (PyCFunction) bits2bytes, METH_O,       bits2bytes_doc},
+    {"_sysinfo",   (PyCFunction) sysinfo,    METH_NOARGS,  sysinfo_doc   },
     {NULL,         NULL}  /* sentinel */
 };
 
