@@ -18,7 +18,7 @@ else:
     from cStringIO import StringIO
 
 
-from bitarray import bitarray, bits2bytes, __version__
+from bitarray import bitarray, bitdiff, bits2bytes, __version__
 
 
 tests = []
@@ -41,7 +41,7 @@ class Util(object):
 
     def randombitarrays(self):
         for n in list(range(25)) + [randint(1000, 2000)]:
-            a = bitarray(endian='big' if randint(0, 1) else 'little')
+            a = bitarray(endian=['little', 'big'][randint(0, 1)])
             a.frombytes(os.urandom(bits2bytes(n)))
             del a[n:]
             yield a
@@ -51,7 +51,10 @@ class Util(object):
             yield [bool(randint(0, 1)) for d in range(n)]
 
     def rndsliceidx(self, length):
-        return randint(-2*length, 2*length-1) if randint(0, 1) == 1 else None
+        if randint(0, 1):
+            return None
+        else:
+            return randint(-2 * length, 2 * length - 1)
 
     def slicelen(self, r, length):
         return getIndicesEx(r, length)[-1]
@@ -86,15 +89,19 @@ def getIndicesEx(r, length):
         if step == 0:
             raise ValueError("slice step cannot be zero")
 
-    defstart = length-1 if step < 0 else 0
-    defstop  = -1 if step < 0 else length
+    if step < 0:
+        defstart = length - 1
+        defstop = -1
+    else:
+        defstart = 0
+        defstop = length
 
     if r.start is None:
         start = defstart
     else:
         if start < 0: start += length
-        if start < 0: start = -1 if step < 0 else 0
-        if start >= length: start = length-1 if step < 0 else length
+        if start < 0: start = [0, -1][step < 0]
+        if start >= length: start = [length, length - 1][step < 0]
 
     if r.stop is None:
         stop = defstop
@@ -106,9 +113,9 @@ def getIndicesEx(r, length):
     if (step < 0 and stop >= length) or (step > 0 and start >= stop):
         slicelength = 0
     elif step < 0:
-        slicelength = (stop-start+1) / step + 1
+        slicelength = (stop - start + 1) / step + 1
     else:
-        slicelength = (stop-start-1) / step + 1
+        slicelength = (stop - start - 1) / step + 1
 
     if slicelength < 0:
         slicelength = 0
@@ -119,6 +126,26 @@ def getIndicesEx(r, length):
 
 class TestsModuleFunctions(unittest.TestCase, Util):
 
+    def test_bitdiff(self):
+        a = bitarray('0011')
+        b = bitarray('0101')
+        self.assertEqual(bitdiff(a, b), 2)
+        self.assertRaises(TypeError, bitdiff, a, '')
+        self.assertRaises(TypeError, bitdiff, '1', b)
+        self.assertRaises(TypeError, bitdiff, a, 4)
+        b.append(1)
+        self.assertRaises(ValueError, bitdiff, a, b)
+
+        for n in list(range(50)) + [randint(1000, 2000)]:
+            a = bitarray()
+            a.frombytes(os.urandom(bits2bytes(n)))
+            del a[n:]
+            b = bitarray()
+            b.frombytes(os.urandom(bits2bytes(n)))
+            del b[n:]
+            diff = sum(a[i] ^ b[i] for i in range(n))
+            self.assertEqual(bitdiff(a, b), diff)
+
     def test_bits2bytes(self):
         for arg in ['foo', [], None, {}]:
             self.assertRaises(TypeError, bits2bytes, arg)
@@ -129,9 +156,9 @@ class TestsModuleFunctions(unittest.TestCase, Util):
         self.assertRaises(ValueError, bits2bytes, -1)
         self.assertRaises(ValueError, bits2bytes, -924)
 
-        for n in range(1000):
-            self.assertEqual(bits2bytes(n),
-                             0 if n==0 else ((n - 1) // 8 + 1));
+        self.assertEqual(bits2bytes(0), 0)
+        for n in range(1, 1000):
+            self.assertEqual(bits2bytes(n), (n - 1) // 8 + 1)
 
         for n, m in [(0, 0), (1, 1), (2, 1), (7, 1), (8, 1), (9, 2),
                      (10, 2), (15, 2), (16, 2), (64, 8), (65, 9),
@@ -245,7 +272,7 @@ class CreateObjectTests(unittest.TestCase, Util):
 
         for n in range(50):
             lst = [bool(randint(0, 1)) for d in range(n)]
-            s = ''.join('1' if x else '0' for x in lst)
+            s = ''.join([['0', '1'][x] for x in lst])
             a = bitarray(s)
             self.assertEqual(a.tolist(), lst)
             self.check_obj(a)
@@ -301,6 +328,35 @@ tests.append(CreateObjectTests)
 
 # ---------------------------------------------------------------------------
 
+class ToObjectsTests(unittest.TestCase, Util):
+
+    def test_int(self):
+        a = bitarray()
+        self.assertRaises(TypeError, int, a)
+        if not is_py3k:
+            self.assertRaises(TypeError, long, a)
+
+    def test_float(self):
+        a = bitarray()
+        self.assertRaises(TypeError, float, a)
+
+    def test_complext(self):
+        a = bitarray()
+        self.assertRaises(TypeError, complex, a)
+
+    def test_list(self):
+        for a in self.randombitarrays():
+            self.assertEqual(list(a), a.tolist())
+
+    def test_tuple(self):
+        for a in self.randombitarrays():
+            self.assertEqual(tuple(a), tuple(a.tolist()))
+
+
+tests.append(ToObjectsTests)
+
+# ---------------------------------------------------------------------------
+
 class MetaDataTests(unittest.TestCase):
 
     def test_buffer_info1(self):
@@ -459,7 +515,7 @@ class SliceTests(unittest.TestCase, Util):
         for a in self.randombitarrays():
             la = len(a)
             if la == 0: continue
-            for dum in range(3):
+            for dum in range(50):
                 step = self.rndsliceidx(la)
                 if step == 0: step = None
                 s = slice(self.rndsliceidx(la),
@@ -518,7 +574,7 @@ class SliceTests(unittest.TestCase, Util):
         for a in self.randombitarrays():
             la = len(a)
             if la == 0: continue
-            for dum in range(10):
+            for dum in range(50):
                 step = self.rndsliceidx(la)
                 if step == 0: step = None
                 s = slice(self.rndsliceidx(la),
@@ -704,6 +760,9 @@ class SpecialMethodTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertTrue(a.all())
 
+        if sys.version_info[:2] < (2, 5):
+            return
+
         for a in self.randombitarrays():
             self.assertEqual(all(a),          a.all())
             self.assertEqual(all(a.tolist()), a.all())
@@ -713,6 +772,9 @@ class SpecialMethodTests(unittest.TestCase, Util):
         a = bitarray()
         self.assertFalse(a.any())
 
+        if sys.version_info[:2] < (2, 5):
+            return
+
         for a in self.randombitarrays():
             self.assertEqual(any(a),          a.any())
             self.assertEqual(any(a.tolist()), a.any())
@@ -1144,11 +1206,29 @@ class ExtendTests(unittest.TestCase, Util):
             for b in self.randomlists():
                 c = bitarray(a)
                 idc = id(c)
-                c.extend(''.join(('1' if x else '0') for x in b))
+                c.extend(''.join(['0', '1'][x] for x in b))
                 self.assertEqual(id(c), idc)
                 self.assertEqual(c.tolist(), a + b)
                 self.check_obj(c)
 
+    def test_extend_self(self):
+        a = bitarray()
+        a.extend(a)
+        self.assertEqual(a, bitarray())
+
+        a = bitarray('1')
+        a.extend(a)
+        self.assertEqual(a, bitarray('11'))
+
+        a = bitarray('110')
+        a.extend(a)
+        self.assertEqual(a, bitarray('110110'))
+
+        for a in self.randombitarrays():
+            b = bitarray(a)
+            a.extend(a)
+            self.assertEqual(a, b + b)
+
 
 tests.append(ExtendTests)
 
@@ -1670,6 +1750,8 @@ class FileTests(unittest.TestCase, Util):
                 self.assertEQUAL(a, b)
 
     def test_shelve(self):
+        if sys.version_info[:2] < (2, 5):
+            return
         import shelve, hashlib
 
         d = shelve.open(self.tmpfname)
@@ -2080,9 +2162,9 @@ if sys.version_info[:2] == (2, 7):
 # ---------------------------------------------------------------------------
 
 def run(verbosity=1, repeat=1):
-    print('bitarray is installed in: ' + os.path.dirname(__file__))
-    print('bitarray version: ' + __version__)
-    print(sys.version)
+    print('bitarray is installed in: %s' % os.path.dirname(__file__))
+    print('bitarray version: %s' % __version__)
+    print('Python version: %s' % sys.version)
 
     suite = unittest.TestSuite()
     for cls in tests:
